package com.sunyard.chsm.service;

import com.sunyard.chsm.auth.UserContext;
import com.sunyard.chsm.enums.KeyAlg;
import com.sunyard.chsm.enums.KeyCategory;
import com.sunyard.chsm.enums.KeyStatus;
import com.sunyard.chsm.enums.KeyUsage;
import com.sunyard.chsm.mapper.AppCertMapper;
import com.sunyard.chsm.mapper.KeyInfoMapper;
import com.sunyard.chsm.mapper.SpKeyRecordMapper;
import com.sunyard.chsm.model.entity.AppCert;
import com.sunyard.chsm.model.entity.KeyInfo;
import com.sunyard.chsm.model.entity.KeyRecord;
import com.sunyard.chsm.param.*;
import com.sunyard.chsm.sdf.SdfApiService;
import com.sunyard.chsm.sdf.model.EccCipher;
import com.sunyard.chsm.sdf.model.EccSignature;
import com.sunyard.chsm.utils.CodecUtils;
import com.sunyard.chsm.utils.gm.BCECUtils;
import com.sunyard.chsm.utils.gm.BCSM2Utils;
import com.sunyard.chsm.utils.gm.BCSM4Utils;
import com.sunyard.chsm.utils.gm.cert.BCSM2CertUtils;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.DERNull;
import org.bouncycastle.asn1.DEROctetString;
import org.bouncycastle.asn1.DERSet;
import org.bouncycastle.asn1.cms.*;
import org.bouncycastle.asn1.gm.GMObjectIdentifiers;
import org.bouncycastle.asn1.x500.X500Name;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.cert.X509CertificateHolder;
import org.bouncycastle.cert.jcajce.JcaCertStore;
import org.bouncycastle.cert.jcajce.JcaX509CertificateConverter;
import org.bouncycastle.cms.*;
import org.bouncycastle.cms.jcajce.JcaSignerInfoGeneratorBuilder;
import org.bouncycastle.cms.jcajce.JcaSignerInfoVerifierBuilder;
import org.bouncycastle.cms.jcajce.JceKeyTransEnvelopedRecipient;
import org.bouncycastle.crypto.params.ECPrivateKeyParameters;
import org.bouncycastle.jcajce.io.CipherInputStream;
import org.bouncycastle.jcajce.provider.asymmetric.ec.BCECPrivateKey;
import org.bouncycastle.jcajce.provider.asymmetric.ec.BCECPublicKey;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.operator.InputDecryptor;
import org.bouncycastle.operator.jcajce.JcaContentSignerBuilder;
import org.bouncycastle.operator.jcajce.JcaDigestCalculatorProviderBuilder;
import org.bouncycastle.util.Store;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;
import java.io.InputStream;
import java.security.Key;
import java.security.cert.X509Certificate;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.Collections;
import java.util.Objects;

/**
 * @author liulu
 * @since 2024/12/20
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class AsymKeyService {

    private final AppCertMapper appCertMapper;
    private final KeyInfoMapper keyInfoMapper;
    private final SpKeyRecordMapper spKeyRecordMapper;
    private final SdfApiService sdfApiService;


    public AsymEncryptResp encrypt(AsymEncryptReq req) {
        byte[] plainData = CodecUtils.decodeBase64(req.getPlainData());
        KeyInfo keyInfo = checkKey(req.getKeyId(), KeyUsage.ENCRYPT_DECRYPT);
        KeyAlg keyAlg = KeyAlg.of(keyInfo.getKeyAlg());
        Assert.notNull(keyAlg, "数据异常");
        KeyRecord keyRecord = spKeyRecordMapper.selectUsedByKeyId(keyInfo.getId());
        Assert.notNull(keyRecord, "数据异常");
        byte[] cipherData;
        switch (keyAlg) {
            case SM2:
                byte[] pub = CodecUtils.decodeHex(keyRecord.getPubKey());
                EccCipher eccCipher = sdfApiService.externalEncryptECC(pub, plainData);
                cipherData = eccCipher.getC1C3C2Bytes();
                break;
            default:
                throw new UnsupportedOperationException("不支持的密钥算法:" + keyAlg.getCode());
        }

        AsymEncryptResp resp = new AsymEncryptResp();
        resp.setKeyId(keyInfo.getId());
        resp.setKeyIndex(keyRecord.getKeyIndex());
        resp.setCipherData(CodecUtils.encodeBase64(cipherData));
        return resp;
    }

    public AsymDecryptResp decrypt(AsymDecryptReq req) {
        byte[] cipherData = CodecUtils.decodeBase64(req.getCipherData());
        KeyInfo keyInfo = checkKey(req.getKeyId(), KeyUsage.ENCRYPT_DECRYPT);
        KeyAlg keyAlg = KeyAlg.of(keyInfo.getKeyAlg());
        Assert.notNull(keyAlg, "数据异常");
        KeyRecord keyRecord = spKeyRecordMapper.selectById(Long.valueOf(req.getKeyIndex()));
        Assert.notNull(keyRecord, "数据异常");
        Assert.isTrue(Objects.equals(keyRecord.getKeyId(), keyInfo.getId()), "密钥Id和密钥索引不匹配");
        byte[] plainData;
        switch (keyAlg) {
            case SM2:
                byte[] encPri = CodecUtils.decodeHex(keyRecord.getKeyData());
                byte[] pri = sdfApiService.decryptByTMK(encPri);
                plainData = sdfApiService.externalDecryptECC(pri, cipherData);
                break;
            default:
                throw new UnsupportedOperationException("不支持的密钥算法:" + keyAlg.getCode());
        }
        AsymDecryptResp resp = new AsymDecryptResp();
        resp.setKeyId(keyInfo.getId());
        resp.setKeyIndex(keyRecord.getKeyIndex());
        resp.setPlainData(CodecUtils.encodeBase64(plainData));
        return resp;
    }

    public AsymSignRawResp signRaw(AsymSignRawReq req) {
        byte[] plainData = CodecUtils.decodeBase64(req.getPlainData());
        byte[] userId = CodecUtils.decodeBase64(req.getUserId());
        KeyInfo keyInfo = checkKey(req.getKeyId(), KeyUsage.SIGN_VERIFY);
        KeyAlg keyAlg = KeyAlg.of(keyInfo.getKeyAlg());
        Assert.notNull(keyAlg, "数据异常");
        KeyRecord keyRecord = spKeyRecordMapper.selectUsedByKeyId(keyInfo.getId());
        Assert.notNull(keyRecord, "数据异常");
        byte[] signData;
        switch (keyAlg) {
            case SM2:
                byte[] encPri = CodecUtils.decodeHex(keyRecord.getKeyData());
                byte[] pri = sdfApiService.decryptByTMK(encPri);
                EccSignature signature = req.isPreProcess()
                        ? sdfApiService.externalSignWithIdECC(pri, plainData, userId)
                        : sdfApiService.externalSignECC(pri, plainData);
                signData = signature.getRawSignBytes();
                break;
            default:
                throw new UnsupportedOperationException("不支持的密钥算法:" + keyAlg.getCode());
        }

        AsymSignRawResp resp = new AsymSignRawResp();
        resp.setKeyId(keyInfo.getId());
        resp.setKeyIndex(keyRecord.getKeyIndex());
        resp.setSignData(CodecUtils.encodeBase64(signData));
        return resp;
    }


    public VerifyResp verifyRaw(AsymVerifyRawReq req) {
        byte[] plainData = CodecUtils.decodeBase64(req.getPlainData());
        byte[] signData = CodecUtils.decodeBase64(req.getSignData());
        byte[] userId = CodecUtils.decodeBase64(req.getUserId());
        KeyInfo keyInfo = checkKey(req.getKeyId(), KeyUsage.SIGN_VERIFY);
        KeyAlg keyAlg = KeyAlg.of(keyInfo.getKeyAlg());
        Assert.notNull(keyAlg, "数据异常");
        KeyRecord keyRecord = spKeyRecordMapper.selectById(Long.valueOf(req.getKeyIndex()));
        Assert.notNull(keyRecord, "数据异常");
        Assert.isTrue(Objects.equals(keyRecord.getKeyId(), keyInfo.getId()), "密钥Id和密钥索引不匹配");
        boolean verified;
        switch (keyAlg) {
            case SM2:
                byte[] pub = CodecUtils.decodeHex(keyRecord.getPubKey());
                verified = req.isPreProcess()
                        ? sdfApiService.externalVerifyWithIdECC(pub, plainData, signData, userId)
                        : sdfApiService.externalVerifyECC(pub, plainData, signData);
                break;
            default:
                throw new UnsupportedOperationException("不支持的密钥算法:" + keyAlg.getCode());
        }
        VerifyResp resp = new VerifyResp();
        resp.setVerified(verified);
        return resp;
    }

    public AsymSignP7Resp signP7Attach(AsymSignP7Req req) {
        byte[] plainData = CodecUtils.decodeBase64(req.getPlainData());
        AppCert appCert = appCertMapper.selectSignBySubject(req.getSubject());
        Assert.notNull(appCert, "证书主题对应的证书不存在");
        Assert.isTrue(Objects.equals(appCert.getApplicationId(), UserContext.getCurrentAppId()), "您无权使用此密钥ID");
        byte[] encPri = CodecUtils.decodeHex(appCert.getEncPriKey());
        byte[] pri = sdfApiService.decryptByTMK(encPri);

        byte[] asymSignP7Resp = p7Sign(pri, appCert.getCertText(), plainData, true);
        AsymSignP7Resp resp = new AsymSignP7Resp();
        resp.setSignData(CodecUtils.encodeBase64(asymSignP7Resp));
        return resp;
    }

    public VerifyResp verifyP7Attach(AsymVerifyP7Req req) {
        byte[] signData = CodecUtils.decodeBase64(req.getSignData());
        boolean verify;
        try {
            verify = p7Verify(signData, null);

        } catch (Exception e) {
            verify = false;
        }
        VerifyResp resp = new VerifyResp();
        resp.setVerified(verify);
        return resp;
    }

    public AsymSignP7Resp signP7Detach(AsymSignP7Req req) {
        byte[] plainData = CodecUtils.decodeBase64(req.getPlainData());
        AppCert appCert = appCertMapper.selectSignBySubject(req.getSubject());
        Assert.notNull(appCert, "证书主题对应的证书不存在");
        Assert.isTrue(Objects.equals(appCert.getApplicationId(), UserContext.getCurrentAppId()), "您无权使用此密钥ID");
        byte[] encPri = CodecUtils.decodeHex(appCert.getEncPriKey());
        byte[] pri = sdfApiService.decryptByTMK(encPri);

        byte[] asymSignP7Resp = p7Sign(pri, appCert.getCertText(), plainData, false);
        AsymSignP7Resp resp = new AsymSignP7Resp();
        resp.setSignData(CodecUtils.encodeBase64(asymSignP7Resp));
        return resp;
    }

    public VerifyResp verifyP7Detach(AsymVerifyP7Req req) {
        Assert.hasText(req.getPlainData(), "P7Detach验签原文不能为空");
        byte[] signData = CodecUtils.decodeBase64(req.getSignData());
        byte[] plainData = CodecUtils.decodeBase64(req.getPlainData());
        boolean verify;
        try {
            verify = p7Verify(signData, plainData);

        } catch (Exception e) {
            verify = false;
        }
        VerifyResp resp = new VerifyResp();
        resp.setVerified(verify);
        return resp;
    }

    private byte[] p7Sign(byte[] pri, String cert, byte[] plainData, boolean encapsulate) {
        try {
            BCECPrivateKey privateKey = BCECUtils.createPrivateKey(pri);
            X509Certificate x509Cert = BCSM2CertUtils.getX509Cert(cert);
            // 构造签名内容
            CMSTypedData cmsData = new CMSProcessableByteArray(plainData);
            // 生成签名者信息
            SignerInfoGenerator signerInfoGenerator = new JcaSignerInfoGeneratorBuilder(
                    new JcaDigestCalculatorProviderBuilder().setProvider(BouncyCastleProvider.PROVIDER_NAME).build()
            ).build(
                    new JcaContentSignerBuilder("SM3withSM2").setProvider(BouncyCastleProvider.PROVIDER_NAME).build(privateKey),
                    x509Cert
            );
            // 构建 CMS Signed Data
            CMSSignedDataGenerator generator = new CMSSignedDataGenerator();
            generator.addSignerInfoGenerator(signerInfoGenerator);
            generator.addCertificates(new JcaCertStore(Collections.singletonList(x509Cert)));
            CMSSignedData signedData = generator.generate(cmsData, encapsulate);
            return signedData.getEncoded();
        } catch (Exception ex) {
            log.warn("", ex);
            throw new IllegalArgumentException("P7Attach 签名异常");
        }
    }

    public static boolean p7Verify(byte[] signedDataBytes, byte[] originalData) throws Exception {

        CMSSignedData signedData;
        if (originalData == null || originalData.length == 0) {
            signedData = new CMSSignedData(signedDataBytes);
        } else {
            CMSTypedData originalContent = new CMSProcessableByteArray(originalData);
            signedData = new CMSSignedData(originalContent, signedDataBytes);
        }

        Store<X509CertificateHolder> certStore = signedData.getCertificates();
        SignerInformationStore signers = signedData.getSignerInfos();

        for (SignerInformation signer : signers.getSigners()) {
            Collection<X509CertificateHolder> matches = certStore.getMatches(signer.getSID());
            if (matches.isEmpty()) {
                throw new IllegalArgumentException("No matching certificate found for signer");
            }
            X509CertificateHolder certHolder = matches.iterator().next(); // 这里进行类型安全的提取
            X509Certificate cert = new JcaX509CertificateConverter()
                    .setProvider(BouncyCastleProvider.PROVIDER_NAME)
                    .getCertificate(certHolder);
            if (signer.verify(new JcaSignerInfoVerifierBuilder(
                    new JcaDigestCalculatorProviderBuilder().setProvider(BouncyCastleProvider.PROVIDER_NAME).build()).build(cert))) {
                return true;
            }
        }
        return false;
    }

    @SneakyThrows
    private byte[] makeEnvelopeData(byte[] srcMsg, byte[] certData) {
        X509Certificate cert = BCSM2CertUtils.getX509Certificate(certData);
        AlgorithmIdentifier symAlg = new AlgorithmIdentifier(GMObjectIdentifiers.sm_scheme.branch("104")); // sm4
        AlgorithmIdentifier asymAlg = new AlgorithmIdentifier(GMObjectIdentifiers.sm2encrypt, DERNull.INSTANCE);

        byte[] symKey = sdfApiService.generateRandom(16);

        BCECPublicKey publicKey = (BCECPublicKey) cert.getPublicKey();
        byte[] encryptKey = BCSM2Utils.encrypt(publicKey, symKey);

        X500Name x500Name = X500Name.getInstance(cert.getIssuerX500Principal().getEncoded());
        KeyTransRecipientInfo keyTransRecipientInfo = new KeyTransRecipientInfo(
                new RecipientIdentifier(new IssuerAndSerialNumber(x500Name, cert.getSerialNumber())),
                asymAlg,
                new DEROctetString(encryptKey)
        );

        byte[] encContent = BCSM4Utils.encrypt_ECB_Padding(symKey, srcMsg);
        RecipientInfo recipientInfo = new RecipientInfo(keyTransRecipientInfo);
        EncryptedContentInfo encContentInfo = new EncryptedContentInfo(
                new ASN1ObjectIdentifier("1.2.156.10197.6.1.4.2.1"),
                symAlg,
                new DEROctetString(encContent)
        );
        EnvelopedData envelopedData = new EnvelopedData(null,
                new DERSet(recipientInfo),
                encContentInfo,
                (org.bouncycastle.asn1.ASN1Set) null);

        ContentInfo contentInfo = new ContentInfo(new ASN1ObjectIdentifier("1.2.156.10197.6.1.4.2.3"), envelopedData);
        return contentInfo.getEncoded("DER");
    }

    @SneakyThrows
    private byte[] decryptEnvelopeData(byte[] envelopeData, byte[] priKeyD) {
        ECPrivateKeyParameters priKeyParameters = BCECUtils.createECPrivateKeyParameters(priKeyD);
        final BCECPrivateKey priKey = new BCECPrivateKey("EC", priKeyParameters, BouncyCastleProvider.CONFIGURATION);

        // 解密数字信封
        CMSEnvelopedData ed = new CMSEnvelopedData(envelopeData);
        RecipientInformationStore recipientInfos = ed.getRecipientInfos();
        Collection<RecipientInformation> recipients = recipientInfos.getRecipients();
        RecipientInformation next = recipients.iterator().next();

        return next.getContent(new JceKeyTransEnvelopedRecipient(priKey) {
            @Override
            @SneakyThrows
            public RecipientOperator getRecipientOperator(AlgorithmIdentifier keyEncAlg,
                                                          AlgorithmIdentifier contentEncryptionAlgorithm,
                                                          byte[] encryptedContentKey) {
                String keyEncId = keyEncAlg.getAlgorithm().getId();
                if (Objects.equals(GMObjectIdentifiers.sm2encrypt.getId(), keyEncId)) {
                    byte[] keyBytes = BCSM2Utils.decrypt(priKeyParameters, encryptedContentKey);
                    Cipher cipher = Cipher.getInstance("SM4/ECB/PKCS7Padding", BouncyCastleProvider.PROVIDER_NAME);
                    Key sm4Key = new SecretKeySpec(keyBytes, "SM4");
                    cipher.init(Cipher.DECRYPT_MODE, sm4Key);
                    return new RecipientOperator(new InputDecryptor() {
                        @Override
                        public AlgorithmIdentifier getAlgorithmIdentifier() {
                            return contentEncryptionAlgorithm;
                        }

                        @Override
                        public InputStream getInputStream(InputStream dataIn) {
                            return new CipherInputStream(dataIn, cipher);
                        }
                    });
                }
                return super.getRecipientOperator(keyEncAlg, contentEncryptionAlgorithm, encryptedContentKey);
            }
        }.setProvider(BouncyCastleProvider.PROVIDER_NAME));
    }


    private KeyInfo checkKey(Long keyId, KeyUsage usage) {

        KeyInfo keyInfo = keyInfoMapper.selectById(keyId);
        Assert.notNull(keyInfo, "密钥ID不存在");
        Assert.isTrue(Objects.equals(keyInfo.getApplicationId(), UserContext.getCurrentAppId()), "您无权使用此密钥ID");
        Assert.isTrue(KeyCategory.ASYM_KEY.getCode().equals(keyInfo.getKeyType()), "此密钥不是对称密钥");

        KeyStatus status = KeyStatus.of(keyInfo.getStatus());
        LocalDateTime now = LocalDateTime.now();
        Assert.isTrue(KeyStatus.ENABLED == status, "此密钥不是启用状态, 无法操作");
        Assert.isTrue(now.isAfter(keyInfo.getEffectiveTime()) && now.isBefore(keyInfo.getExpiredTime()), "此密钥不是启用状态, 无法操作");
        Assert.isTrue(KeyUsage.hasUsage(keyInfo.getKeyUsage(), usage), "此密钥无权进行" + usage.getDesc() + "操作");

        return keyInfo;
    }

}
